package cn.iocoder.yudao.module.member.api.user;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.CustomerMapper;
import cn.iocoder.yudao.module.customer.service.customerOperateLog.CustomerOperateLogService;
import cn.iocoder.yudao.module.customer.vo.customerOperateLog.CustomerOperateLogCreateReqVO;
import cn.iocoder.yudao.module.ecw.enums.CustomerOperateTypeEnum;
import cn.iocoder.yudao.module.member.api.user.dto.UserRespDTO;
import cn.iocoder.yudao.module.member.api.user.dto.UserSelectReqVO;
import cn.iocoder.yudao.module.member.convert.user.UserConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.enums.UserOperationLogTypeEnum;
import cn.iocoder.yudao.module.member.service.auth.MemberAuthService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.member.service.userOperationLog.UserOperationLogService;
import cn.iocoder.yudao.module.member.vo.userOperationLog.UserOperationLogCreateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.util.servlet.ServletUtils.getClientIP;

/**
 * 会员用户的 API 实现类
 *
 * @author 捷道源码
 */
@Service
@Validated
public class MemberUserApiImpl implements MemberUserApi {

    @Resource
    private MemberUserService userService;

    @Resource
    private MemberAuthService memberAuthService;

    @Resource
    private UserOperationLogService userOperationLogService;

    @Resource
    private CustomerMapper customerMapper;

    @Resource
    private CustomerOperateLogService customerOperateLogService;

    @Override
    public UserRespDTO getUser(Long id) {
        MemberUserDO user = userService.info(id);
        return UserConvert.INSTANCE.convert2(user);
    }

    @Override
    public UserRespDTO getUserByMobile(String mobile) {
        MemberUserDO user = userService.getUserByMobile(mobile);
        return UserConvert.INSTANCE.convert2(user);
    }

    @Override
    public void createUserOperationLog(String mobile, UserOperationLogTypeEnum typeEnum, String content) {
        // 检验用户是否存在
        MemberUserDO userDO = memberAuthService.checkUserIfExists(mobile);
        //记录日志
        UserOperationLogCreateReqVO userOperationLogCreateReqVO = new UserOperationLogCreateReqVO();
        userOperationLogCreateReqVO
                .setUserId(userDO.getId())
                .setType(typeEnum.getValue())
                .setTitle(typeEnum.getDesc())
                .setIp(getClientIP())
                .setContent(content);
        userOperationLogService.createUserOperationLog(userOperationLogCreateReqVO);
        //通过memberUserId获取Customer
        CustomerDO customer = customerMapper.getByMemberUserId(userDO.getId());
        if (customer!=null) {
            CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                    .setOperator(1l)
                    .setOperatorName("admin")
                    .setCustomerId(customer.getId())
                    .setNumber(customer.getNumber())
                    .setName(customer.getName())
                    .setOperateType(CustomerOperateTypeEnum.VERIFICATION_CODE.getValue())
                    .setRemark(content);
            //记录日志
            customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
        }
    }

    @Override
    public PageResult<UserRespDTO> getUserList(UserSelectReqVO userSelectReqVO) {
        PageResult<MemberUserDO> memberUserDOS = userService.selectList(userSelectReqVO);
        return UserConvert.INSTANCE.converUserRespDTO(memberUserDOS);
    }

    @Override
    public void updateMemberUserDealStatus(Long customerId, Integer customerStatus) {
        userService.updateMemberUserDealStatus(customerId, customerStatus);
    }
}
