package cn.iocoder.yudao.module.member.controller.admin.levelDetails;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.member.dal.dataobject.level.MemberUserLevelConfigDO;
import cn.iocoder.yudao.module.member.dal.dataobject.levelOperatLog.MemberUserLevelOperateLogDO;
import cn.iocoder.yudao.module.member.dal.dataobject.memberUserLevelDetail.MemberUserLevelDetailsDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserLevel.MemberUserLevelConfigMapper;
import cn.iocoder.yudao.module.member.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.member.service.levelDetails.MemberUserLevelDetailsService;
import cn.iocoder.yudao.module.member.service.levelOperateLog.MemberUserLevelOperateLogService;
import cn.iocoder.yudao.module.member.vo.userLevelDetails.MemberUserLevelDetailsBackVO;
import cn.iocoder.yudao.module.member.vo.userLevelDetails.MemberUserLevelDetailsQueryVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Date;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 会员等级详情记录")
@RequestMapping("/member/level-details")
public class MemberUserLevelDetailsController {
    @Resource
    private MemberUserLevelDetailsService memberUserLevelDetailsService;
    @Resource
    private MemberUserLevelOperateLogService memberUserLevelOperateLogService;
    @Resource
    private MemberUserLevelConfigMapper memberUserLevelConfigMapper;

    @PostMapping("/page")
    @ApiOperation("获得等级详情分页")
    //@PreAuthorize("@ss.hasPermission('member:user-level-log:query')")
    public CommonResult<PageResult<MemberUserLevelDetailsBackVO>> page(@Valid @RequestBody MemberUserLevelDetailsQueryVO query) {
        PageResult<MemberUserLevelDetailsBackVO> pageResult = memberUserLevelDetailsService.getPage(query);
        return success(pageResult);
    }

    @PutMapping("/update")
    @ApiOperation("更新等级详情")
    public CommonResult<Boolean> update(@Valid @RequestBody MemberUserLevelDetailsQueryVO reqVO) {

        MemberUserLevelDetailsDO memberUserLevelDetailsDO;
        if (reqVO.getId() != null){
            memberUserLevelDetailsDO = memberUserLevelDetailsService.getById(reqVO.getId());
            if (memberUserLevelDetailsDO == null) {
                return error(ErrorCodeConstants.LEVEL_DETAILs_NOT_EXISTS);
            }
        }else {
            memberUserLevelDetailsDO = new MemberUserLevelDetailsDO();
        }
        MemberUserLevelConfigDO configDO;
        String levelConfigName="";
        if (memberUserLevelDetailsDO.getConfigId()!=null){
            configDO = memberUserLevelConfigMapper.selectById(memberUserLevelDetailsDO.getConfigId());
            if (configDO == null) {
                return error(ErrorCodeConstants.LEVEL_RULE_NOT_EXISTS);
            }
            if(!configDO.getLevel().equals(reqVO.getLevel())){
                memberUserLevelDetailsDO.setConfigId(null);
            }else {
                levelConfigName=configDO.getName();
            }
        }
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        memberUserLevelDetailsDO.setLevel(reqVO.getLevel());
        memberUserLevelDetailsDO.setMemberId(reqVO.getMemberId());
        memberUserLevelDetailsDO.setValidityPeriod(reqVO.getValidityPeriod());
        memberUserLevelDetailsDO.setSpecificSettings(reqVO.getSpecificSettings());
        memberUserLevelDetailsDO.setRemarks(reqVO.getRemarks());
        memberUserLevelDetailsDO.setUpdateTime(new Date());
        memberUserLevelDetailsDO.setUpdater(String.valueOf(loginUser.getId()));
        memberUserLevelDetailsService.saveOrUpdate(memberUserLevelDetailsDO);
        //添加操作日志

        MemberUserLevelOperateLogDO memberUserLevelOperateLogDo = MemberUserLevelOperateLogDO.builder()
                .configId(memberUserLevelDetailsDO.getConfigId())
                .configName(levelConfigName)
                .memberId(memberUserLevelDetailsDO.getMemberId())
                .operator(loginUser.getNickname())
                .remarks(memberUserLevelDetailsDO.getRemarks())
                .level(memberUserLevelDetailsDO.getLevel())
                .operateType(3)
                .build();
        memberUserLevelOperateLogService.save(memberUserLevelOperateLogDo);
        return success(true);
    }
}
