package cn.iocoder.yudao.module.member.controller.app.address;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.module.member.convert.address.UserAddressConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.address.UserAddressDO;
import cn.iocoder.yudao.module.member.service.address.UserAddressService;
import cn.iocoder.yudao.module.member.vo.address.UserAddressBackVO;
import cn.iocoder.yudao.module.member.vo.address.UserAddressCreateReqVO;
import cn.iocoder.yudao.module.member.vo.address.UserAddressUpdateReqVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "用户 APP - 会员收货地址")
@RequestMapping("/member/user-address")
public class UserAddressController {

    @Resource
    private UserAddressService userAddressService;

    @PostMapping("/create")
    @ApiOperation("创建会员收货地址")
    @Idempotent(timeout = 5)
    //@PreAuthenticated
    public CommonResult<Long> createUserAddress(@Valid @RequestBody UserAddressCreateReqVO createReqVO) {
        return success(userAddressService.createUserAddress(createReqVO));
    }

    @PostMapping("/update")
    @ApiOperation("更新会员收货地址")
    //@PreAuthenticated
    public CommonResult<Boolean> updateUserAddress(@Valid @RequestBody UserAddressUpdateReqVO updateReqVO) {
        userAddressService.updateUserAddress(updateReqVO);
        return success(true);
    }

    @PostMapping("/delete")
    @ApiOperation("删除会员收货地址")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthenticated
    public CommonResult<Boolean> deleteUserAddress(@RequestParam("id") Long id) {
        userAddressService.deleteUserAddress(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("根据id获得会员收货地址")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthenticated
    public CommonResult<UserAddressBackVO> getUserAddress(@RequestParam("id") Long id) {
        UserAddressDO userAddress = userAddressService.getUserAddress(id);
        return success(UserAddressConvert.INSTANCE.convert(userAddress));
    }

    @GetMapping("/member/list")
    @ApiOperation("获得会员收货地址列表")
    @ApiImplicitParam(name = "id", value = "会员id", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthenticated
    public CommonResult<List<UserAddressBackVO>> getUserAddressList(@RequestParam("id") Long memberId) {
        List<UserAddressDO> list = userAddressService.getUserAddressListByMemberId(memberId);
        return success(UserAddressConvert.INSTANCE.convertList(list));
    }

    //@GetMapping("/list")
    //@ApiOperation("获得会员收货地址列表")
    //@ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    ////@PreAuthenticated
    //public CommonResult<List<UserAddressBackVO>> getUserAddressList(@RequestParam("ids") Collection<Long> ids) {
    //    List<UserAddressDO> list = userAddressService.getUserAddressList(ids);
    //    return success(UserAddressConvert.INSTANCE.convertList(list));
    //}

    //@GetMapping("/page")
    //@ApiOperation("获得会员收货地址分页")
    //@PreAuthenticated
    //public CommonResult<PageResult<UserAddressBackVO>> getUserAddressPage(@Valid UserAddressQueryVO query, PageVO page) {
    //    PageResult<UserAddressDO> pageResult = userAddressService.getUserAddressPage(query, page);
    //    return success(UserAddressConvert.INSTANCE.convertPage(pageResult));
    //}

    //@GetMapping("/export-excel")
    //@ApiOperation("导出会员收货地址 Excel")
    //@PreAuthenticated
    //@OperateLog(type = EXPORT)
    //public void exportUserAddressExcel(@Valid UserAddressQueryVO query,
    //          HttpServletResponse response) throws IOException {
    //    List<UserAddressDO> list = userAddressService.getUserAddressList(query);
    //    // 导出 Excel
    //    List<UserAddressBackVO> datas = UserAddressConvert.INSTANCE.convertList(list);
    //    ExcelUtils.write(response, "会员收货地址.xls", "数据", UserAddressBackVO.class, datas);
    //}
}
