package cn.iocoder.yudao.module.member.controller.app.auth.vo;

import cn.iocoder.yudao.framework.common.validation.InEnum;
import cn.iocoder.yudao.framework.common.validation.Mobile;
import cn.iocoder.yudao.module.system.enums.social.SocialTypeEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

/**
 * @author Administrator
 */
@ApiModel("用户 APP - 社交登录 Request VO，使用 code 授权码 + 账号密码")
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class AppAuthSocialLogin2ReqVO {

    @ApiModelProperty(value = "社交平台的类型", required = true, example = "10", notes = "参见 SysUserSocialTypeEnum 枚举值")
    @InEnum(value = SocialTypeEnum.class, message = "{app.social.type.not.range}")
    @NotNull(message = "{app.social.type.not.blank}")
    private Integer type;

    @ApiModelProperty(value = "授权码", required = true, example = "1024")
    @NotBlank(message = "{app.social.code.not.blank}")
    private String code;

    @ApiModelProperty(value = "state", required = true, example = "9b2ffbc1-7425-4155-9894-9d5c08541d62")
    @NotBlank(message = "{app.social.state.not.blank}")
    private String state;

    @ApiModelProperty(value = "手机号", required = true, example = "15119100000")
    @NotBlank(message = "{app.auth.mobile.not.blank}")
    @Mobile(message = "{app.auth.mobile.format.err}")
    private String mobile;

    @ApiModelProperty(value = "手机验证码", required = true, example = "1024")
    @NotBlank(message = "{app.auth.captcha.not.blank}")
    @Length(min = 4, max = 6, message = "{app.auth.captcha.len.err}")
    @Pattern(regexp = "^[0-9]+$", message = "{app.auth.captcha.pattern}")
    private String smsCode;
}
