package cn.iocoder.yudao.module.member.controller.app.scoreRule;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.ip.IPHelper;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.module.member.api.ScoreProducerApi;
import cn.iocoder.yudao.module.member.controller.app.scoreRule.vo.AppScoreRuleBackDetailVO;
import cn.iocoder.yudao.module.member.controller.app.scoreRule.vo.AppScoreRuleListBackVO;
import cn.iocoder.yudao.module.member.controller.app.scoreRule.vo.AppScoreRuleListReqVO;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.service.ScoreRuleShareRecord.ScoreRuleShareRecordService;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.vo.scoreRule.IdReqVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.context.ApplicationContext;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;


@Validated
@RestController
@Api(tags = "客户端 - 积分规则")
@RequestMapping("/member/score-rule")
public class AppScoreRuleController {

    @Resource
    private ScoreRuleService scoreRuleService;
    @Resource
    private ApplicationContext applicationContext;
    @Resource
    private ScoreProducerApi scoreProducerApi;
    @Resource
    private ScoreRuleShareRecordService scoreRuleShareRecordService;


    @PostMapping("/get")
    @ApiOperation("客户端获得积分规则详情")
    @Idempotent(timeout = 1)
    public CommonResult<AppScoreRuleBackDetailVO> getScoreRule(@Valid @RequestBody IdReqVo idReqVo) {
        AppScoreRuleBackDetailVO scoreRuleBackDetailVO = scoreRuleService.appGetScoreRule(idReqVo.getId());
        return success(scoreRuleBackDetailVO);
    }

    @PostMapping("/list")
    @ApiOperation("客户端获得积分规则列表")
    @Idempotent(timeout = 1, message = "页面加载中")
    public CommonResult<List<AppScoreRuleListBackVO>> getScoreRuleList(@Valid @RequestBody AppScoreRuleListReqVO reqVO) {
        List<AppScoreRuleListBackVO> list = scoreRuleService.appGetScoreRuleList(reqVO);
        return success(list);
    }

    @GetMapping("/trigger/share")
    @ApiOperation("分享规则触发")
    @Idempotent(timeout = 5)
    public void triggerShareScoreRule(HttpServletRequest request, @RequestParam Long userId, @RequestParam Long ruleId) {
        if (ruleId == null || userId == null) {
            return;
        }
        String ipAddr = IPHelper.getIpAddr(request);
        if (!scoreRuleShareRecordService.verifyTriggerIp(userId, ruleId, ipAddr)) {
            return;
        }
        scoreProducerApi.sendShareMessage(ScoreRuleTypeEnum.SHARE, userId, ruleId, ipAddr);
    }
    //@PostMapping("/page")
    //@ApiOperation("获得积分规则分页")
    //public CommonResult<PageResult<ScoreRuleBackVO>> getScoreRulePage(@Valid @RequestBody ScoreRuleQueryVO query, PageVO page) {
    //    PageResult<ScoreRuleBackVO> pageResult = scoreRuleService.getScoreRulePage(query, page);
    //    return success(pageResult);
    //}
}
