package cn.iocoder.yudao.module.member.controller.app.user;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.ecw.api.paramValid.ParamValidatorApi;
import cn.iocoder.yudao.module.ecw.dal.dataobject.country.CountryDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.ecw.service.internalMessage.WebInternalMessageService;
import cn.iocoder.yudao.module.ecw.service.region.RegionService;
import cn.iocoder.yudao.module.member.controller.app.auth.vo.AppEmailUpdateReqVO;
import cn.iocoder.yudao.module.member.controller.app.user.vo.*;
import cn.iocoder.yudao.module.member.controller.app.userScore.vo.AppMemberUserScoreInfoRespVO;
import cn.iocoder.yudao.module.member.convert.user.UserConvert;
import cn.iocoder.yudao.module.member.convert.userCardAuth.UserCardAuthConvert;
import cn.iocoder.yudao.module.member.convert.userEnterprise.UserEnterpriseConvert;
import cn.iocoder.yudao.module.member.convert.userEnterpriseAuth.UserEnterpriseAuthConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.dal.dataobject.userCardAuth.UserCardAuthDO;
import cn.iocoder.yudao.module.member.dal.dataobject.userEnterprise.UserEnterpriseDO;
import cn.iocoder.yudao.module.member.dal.dataobject.userEnterpriseAuth.UserEnterpriseAuthDO;
import cn.iocoder.yudao.module.member.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.member.service.level.MemberUserLevelConfigService;
import cn.iocoder.yudao.module.member.service.score.MemberUserScoreService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.member.service.userCardAuth.UserCardAuthService;
import cn.iocoder.yudao.module.member.service.userEnterprise.UserEnterpriseService;
import cn.iocoder.yudao.module.member.service.userEnterpriseAuth.UserEnterpriseAuthService;
import cn.iocoder.yudao.module.member.vo.member.MemberUpdateReqVO;
import cn.iocoder.yudao.module.member.vo.userCardAuth.UserCardAuthBackVO;
import cn.iocoder.yudao.module.member.vo.userCardAuth.UserCardAuthCreateReqVO;
import cn.iocoder.yudao.module.member.vo.userEnterprise.UserEnterpriseBackVO;
import cn.iocoder.yudao.module.member.vo.userEnterprise.UserEnterpriseCreateReqVO;
import cn.iocoder.yudao.module.member.vo.userEnterprise.UserEnterpriseUpdateReqVO;
import cn.iocoder.yudao.module.member.vo.userEnterpriseAuth.UserEnterpriseAuthBackVO;
import cn.iocoder.yudao.module.member.vo.userEnterpriseAuth.UserEnterpriseAuthCreateReqVO;
import cn.iocoder.yudao.module.product.service.coupon.CouponService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.text.SimpleDateFormat;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;
import static cn.iocoder.yudao.module.infra.enums.ErrorCodeConstants.FILE_IS_EMPTY;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.MEMBER_USER_MOBILE_EXIST;

/**
 * @author Administrator
 */
@Api(tags = "用户 APP - 用户个人中心")
@RestController
@RequestMapping("/member/user")
@Validated
@Slf4j
public class AppUserController {

    @Resource
    private MemberUserService userService;
    @Resource
    private CustomerContactsService customerContactsService;
    @Resource
    private CustomerService customerService;
    @Resource
    private UserEnterpriseAuthService userEnterpriseAuthService;

    @Resource
    private UserCardAuthService userCardAuthService;

    @Resource
    private UserEnterpriseService userEnterpriseService;

    @Resource
    private WebInternalMessageService webInternalMessageService;

    @Resource
    private CouponService couponService;
    @Resource
    private ParamValidatorApi paramValidatorApi;

    @Resource
    private PasswordEncoder passwordEncoder;

    @Resource
    private MemberUserLevelConfigService memberUserLevelConfigService;

    @Resource
    private RegionService regionService;

    @Resource
    private MemberUserScoreService memberUserScoreService;


    @PutMapping("/update-nickname")
    @ApiOperation("修改用户昵称")
    @PreAuthenticated
    public CommonResult<Boolean> updateUserNickname(@RequestParam("nickname") String nickname) {
        userService.updateUserNickname(getLoginUserId(), nickname);
        return success(true);
    }

    @PutMapping("/new/update-Avatar")
    @ApiOperation("修改用户头像，新接口")
    @PreAuthenticated
    public CommonResult<Boolean> updateNewAvatar(@RequestParam("avatar") String avatar) {
        userService.updateNewUserAvatar(getLoginUserId(), avatar);
        return success(true);
    }

    @PutMapping("/update-avatar")
    @ApiOperation("修改用户头像")
    @PreAuthenticated
    public CommonResult<String> updateUserAvatar(@RequestParam("avatarFile") MultipartFile file) throws Exception {
        if (file.isEmpty()) {
            throw exception(FILE_IS_EMPTY);
        }
        String avatar = userService.updateUserAvatar(getLoginUserId(), file.getInputStream());
        return success(avatar);
    }


    @GetMapping("/get")
    @ApiOperation("获得基本信息")
    @PreAuthenticated
    public CommonResult<AppUserInfoRespVO> getUserInfo() {
        MemberUserDO user = userService.getUser(getLoginUserId());
        //解决日期不生效问题
        AppUserInfoRespVO respVO = UserConvert.INSTANCE.convert(user);
        if (user.getBirthday() != null) {
            respVO.setBirthday(new SimpleDateFormat("yyyy-MM-dd").format(user.getBirthday()));
        }
        respVO.setIsHavePassword(!StringUtils.isBlank(user.getPassword()));

        //判断密码是否是简单密码
        if (passwordEncoder.matches("123456", user.getPassword())) {
            respVO.setIsSimplePassword(Boolean.TRUE);
        }
        if (passwordEncoder.matches("88888888", user.getPassword())) {
            respVO.setIsSimplePassword(Boolean.TRUE);
        }

        // 根据会员ID获取客户联系人ID与客户ID,并返回部分客户档案设置
        CustomerContactsDO customerContacts =
                customerContactsService.getOne(new LambdaQueryWrapper<CustomerContactsDO>()
                        .and(we -> we.eq(CustomerContactsDO::getUserid, user.getId())
                                .or()
                                .eq(CustomerContactsDO::getPhoneNew, user.getMobile())
                                .eq(CustomerContactsDO::getAreaCode, user.getAreaCode())).eq(CustomerContactsDO::getDeleted, 0).last("limit 1"));
        if (Objects.isNull(customerContacts)) {
            respVO.setDefaultPay(Boolean.FALSE);
            respVO.setNoConsignee(Boolean.FALSE);
        }else {
            CustomerDO customerDO = customerService.getCustomer(customerContacts.getCustomerId());
            if (customerDO != null) {
                respVO.setDefaultPay(customerDO.getDefaultPay());
                respVO.setNoConsignee(customerDO.getNoConsignee());
                respVO.setFirstDealTime(customerDO.getFirstDealTime());
            }else {
                respVO.setDefaultPay(Boolean.FALSE);
                respVO.setNoConsignee(Boolean.FALSE);
            }
        }

        //获取用户积分信息
        AppMemberUserScoreInfoRespVO userScoreInfo = memberUserScoreService.getUserScoreInfo(user.getId());
        respVO.setUserScoreLevelInfo(userScoreInfo);
        if (user.getCountry() != null) {
            RegionDO region = regionService.getRegion(Long.valueOf(user.getCountry()));
            respVO.setCountryTitleEn(region.getTitleEn());
            respVO.setCountryTitleZh(region.getTitleZh());
        }
        if (user.getCity() != null) {
            RegionDO region = regionService.getRegion(Long.valueOf(user.getCity()));
            respVO.setCityTitleEn(region.getTitleEn());
            respVO.setCityTitleZh(region.getTitleZh());
        }
        return success(respVO);
    }

    @GetMapping("/getCompanyInfo")
    @ApiOperation("获得公司基本信息")
    @PreAuthenticated
    public CommonResult<UserEnterpriseBackVO> getCompanyInfo() {
        UserEnterpriseDO userCompanyInfo = userEnterpriseService.getUserEnterprise(getLoginUserId());
        return success(UserEnterpriseConvert.INSTANCE.convert(userCompanyInfo));
    }

    @PostMapping("/createCompanyInfo")
    @ApiOperation("创建会员公司信息")
    @PreAuthenticated
    @Idempotent(timeout = 3)
    public CommonResult<Long> createUserEnterprise(@Valid @RequestBody UserEnterpriseCreateReqVO createReqVO) {
        createReqVO.setUserId(getLoginUserId());
        return success(userEnterpriseService.createUserEnterprise(createReqVO));
    }

    @PutMapping("/updateCompanyInfo")
    @ApiOperation("更新会员公司信息")
    @PreAuthenticated
    public CommonResult<Boolean> updateUserEnterprise(@Valid @RequestBody UserEnterpriseUpdateReqVO updateReqVO) {
        updateReqVO.setUserId(getLoginUserId());
        userEnterpriseService.updateUserEnterprise(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/deleteCompanyInfo")
    @ApiOperation("删除会员公司信息")
    @PreAuthenticated
    public CommonResult<Boolean> deleteUserEnterprise(@RequestParam("id") Long id) {
        userEnterpriseService.deleteUserEnterprise(id);
        return success(true);
    }

    @PostMapping("/update-mobile")
    @ApiOperation(value = "修改用户手机")
    @PreAuthenticated
    public CommonResult<Boolean> updateMobile(@RequestBody @Valid AppUserUpdateMobileReqVO reqVO) {
        String mobileCode = reqVO.getAreaCode() + StrUtil.COLON + reqVO.getMobile();
        paramValidatorApi.validatorMobile(mobileCode);
//        String mobile = PhoneUtil.formatPhone(mobileCode);
//        if (StringUtils.isNotBlank(mobile)) {
//            reqVO.setMobile(mobile);
//        }
        MemberUserDO memberUserDO = userService.getUserByMobile(reqVO.getAreaCode(), reqVO.getMobile());
        if (memberUserDO != null) {
            return error(MEMBER_USER_MOBILE_EXIST);
        }
        userService.updateUserMobile(getLoginUserId(), reqVO);
        return success(true);
    }

    @PostMapping("/bind-email")
    @ApiOperation(value = "绑定邮箱", notes = "用户绑定邮箱使用")
    @PreAuthenticated
    public CommonResult<Boolean> bindEmail(@RequestBody @Valid AppEmailUpdateReqVO reqVO) {
        userService.updateEmail(getLoginUserId(), reqVO);
        return success(true);
    }


    @PutMapping("/auth-idcard")
    @ApiOperation("实名认证")
    @PreAuthenticated
    public CommonResult<Boolean> idcardAuth(@RequestBody @Valid AppUserCardAuthReqVO reqVO) {
        Long loginUserId = getLoginUserId();

        MemberUserDO user = userService.getUser(loginUserId);
        if (user == null) {
            throw exception(ErrorCodeConstants.USER_NOT_EXISTS);
        }

        UserCardAuthCreateReqVO userCardAuthCreateReqVO = UserCardAuthConvert.INSTANCE.convert(reqVO);
        userCardAuthCreateReqVO.setUserId(user.getId());

        userCardAuthService.createUserCardAuth(userCardAuthCreateReqVO, false);

        return success(true);
    }


    @GetMapping("/get-auth-idcard-info")
    @ApiOperation("获取登录用户提交的实名认证信息")
    @PreAuthenticated
    public CommonResult<UserCardAuthBackVO> getIdcardAuthInfo() {

        Long loginUserId = getLoginUserId();

        UserCardAuthDO one = userCardAuthService.getOne(new LambdaQueryWrapperX<UserCardAuthDO>().eq(UserCardAuthDO::getUserId, loginUserId));

        return success(UserCardAuthConvert.INSTANCE.convert(one));
    }


    @PutMapping("/auth-enterprise")
    @ApiOperation("企业认证")
    @PreAuthenticated
    public CommonResult<Boolean> enterpriseAuth(@RequestBody @Valid AppUserEnterpriseAuthReqVO reqVO) {
        Long loginUserId = getLoginUserId();

        MemberUserDO user = userService.getUser(loginUserId);
        if (user == null) {
            return error(ErrorCodeConstants.USER_NOT_EXISTS);
        }
        UserEnterpriseAuthCreateReqVO userEnterpriseAuthCreateReqVO = UserEnterpriseAuthConvert.INSTANCE.convert(reqVO);
        userEnterpriseAuthCreateReqVO.setUserId(loginUserId);

        userEnterpriseAuthService.createUserEnterpriseAuth(userEnterpriseAuthCreateReqVO, false);

        return success(true);
    }

    @GetMapping("/get-auth-enterprise-info")
    @ApiOperation("获取登录用户提交的企业认证信息")
    @PreAuthenticated
    public CommonResult<UserEnterpriseAuthBackVO> getEnterpriseAuthInfo() {

        Long loginUserId = getLoginUserId();

        UserEnterpriseAuthDO one = userEnterpriseAuthService.getOne(new LambdaQueryWrapperX<UserEnterpriseAuthDO>().eq(UserEnterpriseAuthDO::getUserId, loginUserId));

        return success(UserEnterpriseAuthConvert.INSTANCE.convert(one));
    }

    @PutMapping("/update")
    @ApiOperation("更新用户")
    @PreAuthenticated
    public CommonResult<Boolean> updateUser(@Valid @RequestBody MemberUpdateReqVO updateReqVO) {
        userService.updateUser(updateReqVO);
        return success(true);
    }


    @GetMapping("/angle-mark")
    @ApiOperation("获取用户统计信息数量角标")
    @PreAuthenticated
    public CommonResult<AppUserAngleMarkVO> userAngleMark() {
        Long userId = getLoginUserId();
        MemberUserDO user = userService.getUser(userId);
        // 根据会员ID获取客户联系人ID与客户ID
        List<CustomerContactsDO> customerContactsList = customerContactsService.list(new LambdaQueryWrapper<CustomerContactsDO>()
                .eq(CustomerContactsDO::getUserid, userId)
                .eq(CustomerContactsDO::getDeleted, 0));
        Set<Long> customerContactsIds = new HashSet<>();
        if (CollectionUtil.isNotEmpty(customerContactsList)){
            customerContactsIds = customerContactsList.stream().map(CustomerContactsDO::getId).collect(Collectors.toSet());
        }
        AppUserAngleMarkVO markVO = new AppUserAngleMarkVO();
        // 控货中的订单数
        markVO.setCargoControlInNum(userService.appCargoControlCount(user.getId(), user.getMobile(), 2));
        // 控货的全部订单数
        markVO.setCargoControlAllNum(userService.appCargoControlCount(user.getId(), user.getMobile(), 1));
        // 有效的优惠劵数量 TODO 展示先屏蔽掉优惠劵统计
//        markVO.setCouponNum(couponService.countUserCoupon(userId));
        markVO.setCouponNum(0);
        // 未读消息数
        markVO.setUnreadMessageNum(webInternalMessageService.countUserUnreadMessage(userId));
        // TODO 待结款订单数待完善
        markVO.setNotSettlementNum(0);
        // 我发出的订单数
        markVO.setIssuedOrderNum(userService.appIssuedOrderCount(user.getMobile(), userId, customerContactsIds));
        // 我收到的订单数
        markVO.setReceivedOrderNum(userService.appReceivedOrderCount(user.getMobile(), customerContactsIds));
        // 我的全部订单数
        markVO.setAllOrderNum(userService.appAllOrderCount(userId, user.getMobile(), customerContactsIds));
        return success(markVO);
    }
}

