package cn.iocoder.yudao.module.member.dal.mysql.scoreRule;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.member.enums.RelationSymbolEnum;
import cn.iocoder.yudao.module.member.vo.scoreRule.ScoreRuleQueryVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

/**
 * 积分规则 Mapper
 *
 * @author 系统管理员
 */
@Mapper
public interface ScoreRuleMapper extends AbstractMapper<ScoreRuleDO> {
    @Override
    default PageResult<ScoreRuleDO> selectPage(PageVO page, Object object) {
        if (object instanceof ScoreRuleQueryVO) {
            ScoreRuleQueryVO vo = (ScoreRuleQueryVO) object;
            LambdaQueryWrapperX<ScoreRuleDO> queryWrapperX = new LambdaQuery<ScoreRuleDO>()
                    .eqIfPresent(ScoreRuleDO::getType, vo.getType())
                    .eqIfPresent(ScoreRuleDO::getStatus, vo.getStatus())
                    .betweenIfPresent(ScoreRuleDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(ScoreRuleDO::getId);
            if (vo.getGetScoreOnceSymbol() != null && vo.getGetScoreOnce() != null) {
                queryWrapperX.gt(vo.getGetScoreOnceSymbol() == RelationSymbolEnum.GT.getValue(), ScoreRuleDO::getGetScoreOnce, vo.getGetScoreOnce())
                        .eq(vo.getGetScoreOnceSymbol() == RelationSymbolEnum.EQ.getValue(), ScoreRuleDO::getGetScoreOnce, vo.getGetScoreOnce())
                        .lt(vo.getGetScoreOnceSymbol() == RelationSymbolEnum.LT.getValue(), ScoreRuleDO::getGetScoreOnce, vo.getGetScoreOnce());
            }
            if (vo.getMaxScoreTotalSymbol() != null && vo.getMaxScoreTotal() != null) {
                queryWrapperX.gt(vo.getMaxScoreTotalSymbol() == RelationSymbolEnum.GT.getValue(), ScoreRuleDO::getMaxScoreTotal, vo.getMaxScoreTotal())
                        .eq(vo.getMaxScoreTotalSymbol() == RelationSymbolEnum.EQ.getValue(), ScoreRuleDO::getMaxScoreTotal, vo.getMaxScoreTotal())
                        .lt(vo.getMaxScoreTotalSymbol() == RelationSymbolEnum.LT.getValue(), ScoreRuleDO::getMaxScoreTotal, vo.getMaxScoreTotal());
            }
            queryWrapperX.and(vo.getTitle() != null, wrapper ->
                    wrapper.like(ScoreRuleDO::getTitleEn, vo.getTitle())
                            .or()
                            .like(ScoreRuleDO::getTitleZh, vo.getTitle())

            );
            queryWrapperX.and(vo.getDesc() != null, wrapper ->
                    wrapper.like(ScoreRuleDO::getDescZh, vo.getDesc())
                            .or()
                            .like(ScoreRuleDO::getDescEn, vo.getDesc())

            );
            return selectPage(page, queryWrapperX);
        }
        return null;
    }

    @Override
    default List<ScoreRuleDO> selectList(Object object) {
        if (object instanceof ScoreRuleQueryVO) {
            ScoreRuleQueryVO vo = (ScoreRuleQueryVO) object;
            return selectList(new LambdaQuery<ScoreRuleDO>()
                    .eqIfPresent(ScoreRuleDO::getType, vo.getType())
                    .eqIfPresent(ScoreRuleDO::getGetScoreOnce, vo.getGetScoreOnce())
                    .eqIfPresent(ScoreRuleDO::getMaxScoreTotal, vo.getMaxScoreTotal())
                    .eqIfPresent(ScoreRuleDO::getStatus, vo.getStatus())
                    .betweenIfPresent(ScoreRuleDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(ScoreRuleDO::getId));
        }
        return null;
    }
    @ResultType(String.class)
    @Select({"select code from score_rule order by code desc limit 1"})
    String getCurrentMaxScoreRuleCode();
}
