package cn.iocoder.yudao.module.member.dal.redis.scoreRule;

import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.redis.RedisKeyConstant;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.TransportTypeEnum;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleOrderVExtraVO;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;

import static cn.iocoder.yudao.module.member.dal.redis.RedisKeyConstant.MEMBER_USER_SCORE_RULE_SWITCH;

/**
 * 会员积分规则 Redis DAO
 */
@Repository
public class ScoreRuleRedisDao {
    @Resource
    private StringRedisTemplate stringRedisTemplate;

    public Boolean setScoreRuleSwitch(Boolean switchStatus) {
        stringRedisTemplate.opsForValue().set(MEMBER_USER_SCORE_RULE_SWITCH, switchStatus.toString());
        return switchStatus;
    }

    public Boolean getScoreRuleSwitch() {
        String string = stringRedisTemplate.opsForValue().get(MEMBER_USER_SCORE_RULE_SWITCH);
        if (string == null) {
            stringRedisTemplate.opsForValue().set(MEMBER_USER_SCORE_RULE_SWITCH, "false");
            return false;
        }
        return Boolean.parseBoolean(string);
    }

    public ScoreRuleDO getEnableScoreRule(ScoreRuleTypeEnum type, TransportTypeEnum transportType) {
        ScoreRuleDO useScoreRuleDO;
        // 如果transportType不为空则是订单V值类型
        if (transportType == null) {
            String redisKey = formatKey(type.getValue(), null);
            String scoreRuleJson = stringRedisTemplate.opsForValue().get(redisKey);
            if (scoreRuleJson == null) {
                return null;
            }
            useScoreRuleDO = JsonUtils.parseObject(scoreRuleJson, ScoreRuleDO.class);
        } else {
            String redisKey = formatKey(type.getValue(), transportType.getValue());
            String scoreRuleJson = stringRedisTemplate.opsForValue().get(redisKey);
            if (scoreRuleJson == null) {
                return null;
            }
            useScoreRuleDO = JsonUtils.parseObject(scoreRuleJson, ScoreRuleDO.class);
        }
        // 如果当前时间不在规则时间区间
        if (System.currentTimeMillis() < useScoreRuleDO.getStartTime().getTime() || System.currentTimeMillis() > useScoreRuleDO.getEndTime().getTime()) {
            return null;
        }
        return useScoreRuleDO;
    }

    public ScoreRuleDO getEnableScoreRule(ScoreRuleTypeEnum type) {
        return getEnableScoreRule(type, null);
    }

    public void setEnableScoreRule(ScoreRuleDO scoreRuleDO) {
        // 多余字段置空
        scoreRuleDO.setCreator(null);
        scoreRuleDO.setUpdater(null);
        scoreRuleDO.setUpdateTime(null);
        scoreRuleDO.setCreateTime(null);
        scoreRuleDO.setDeleted(null);
        scoreRuleDO.setCoverImageEn(null);
        scoreRuleDO.setCoverImageZh(null);
        scoreRuleDO.setDescEn(null);
        scoreRuleDO.setDescZh(null);
        if (scoreRuleDO.getType() == ScoreRuleTypeEnum.ORDER_V.getValue()) {
            ScoreRuleOrderVExtraVO scoreRuleOrderVExtraVO = JsonUtils.parseObject(scoreRuleDO.getExtra(), ScoreRuleOrderVExtraVO.class);
            stringRedisTemplate.opsForValue().set(formatKey(scoreRuleDO.getType(), scoreRuleOrderVExtraVO.getTransportType()), JsonUtils.toJsonString(scoreRuleDO), 24L, TimeUnit.HOURS);
        }
        stringRedisTemplate.opsForValue().set(formatKey(scoreRuleDO.getType(), null), JsonUtils.toJsonString(scoreRuleDO), 24L, TimeUnit.HOURS);
    }

    public void deleteEnableScoreRule(Integer type) {
        if (ScoreRuleTypeEnum.ORDER_V.getValue() == type) {
            stringRedisTemplate.delete(formatKey(type, 1));
            stringRedisTemplate.delete(formatKey(type, 3));
        }
        String redisKey = formatKey(type, null);
        stringRedisTemplate.delete(redisKey);
    }

    private static String formatKey(Integer type, Integer transportType) {
        if (transportType == null) {
            return RedisKeyConstant.MEMBER_USER_SCORE_RULE + type;
        }
        return RedisKeyConstant.MEMBER_USER_SCORE_RULE + type + ":" + transportType;
    }
}
