package cn.iocoder.yudao.module.member.job;

import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.quartz.core.handler.JobHandler;
import cn.iocoder.yudao.module.member.dal.dataobject.level.MemberUserLevelConfigDO;
import cn.iocoder.yudao.module.member.enums.MemberLevelStatusEnum;
import cn.iocoder.yudao.module.member.service.level.MemberUserLevelConfigService;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 会员等级规则过期定时任务
 * @author Smile
 */
@Component
@Slf4j
public class UserLevelConfigExpireTask implements JobHandler {
    /**
     * 会员等级规则的过期时间为每日的0点整
     * 任务每天0点整运行，扫描当天0点过期的积分进行状态修改
     * @param param 参数
     * @return
     * @throws Exception
     */

    @Resource
    private MemberUserLevelConfigService memberUserLevelConfigService;

    @Override
    public String execute(String param) throws Exception {
        log.info("level config expire task running");
        LambdaQuery<MemberUserLevelConfigDO> userLevelConfigDOLambdaQuery = new LambdaQuery<>();
        userLevelConfigDOLambdaQuery.eq(MemberUserLevelConfigDO::getStatus, MemberLevelStatusEnum.ENABLED.getValue());
        userLevelConfigDOLambdaQuery.le(MemberUserLevelConfigDO::getEndTime, DateUtils.getNextNDayStart(new Date(), 0));
        List<MemberUserLevelConfigDO> todoList = memberUserLevelConfigService.selectList(userLevelConfigDOLambdaQuery);
        log.info("level config expire task, to expire rule count :{}", todoList.size());
        if (CollectionUtils.isEmpty(todoList)) {
            return "success";
        } else {
            todoList.forEach(userLevelConfigDO -> {
                userLevelConfigDO.setStatus(MemberLevelStatusEnum.EXPIRED.getValue());
                memberUserLevelConfigService.updateById(userLevelConfigDO);
            });
        }
        return "success";
    }
}
