package cn.iocoder.yudao.module.member.job;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.quartz.core.handler.JobHandler;
import cn.iocoder.yudao.module.member.dal.dataobject.level.MemberUserLevelConfigDO;
import cn.iocoder.yudao.module.member.dal.dataobject.levelOperatLog.MemberUserLevelOperateLogDO;
import cn.iocoder.yudao.module.member.dal.dataobject.memberUserLevelDetail.MemberUserLevelDetailsDO;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.enums.MemberLevelStatusEnum;
import cn.iocoder.yudao.module.member.service.level.MemberUserLevelConfigService;
import cn.iocoder.yudao.module.member.service.levelDetails.MemberUserLevelDetailsService;
import cn.iocoder.yudao.module.member.service.levelOperateLog.MemberUserLevelOperateLogService;
import cn.iocoder.yudao.module.member.service.scoreLog.MemberUserScoreLogService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.member.vo.userLevelDetails.MemberUserLevelDetailsBackVO;
import cn.iocoder.yudao.module.member.vo.userLevelDetails.MemberUserLevelDetailsQueryVO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 会员等级详情定时任务
 */
@Component
@Slf4j
public class userLevelDefaultDetailsTask implements JobHandler {
    /**
     * 每天凌晨00:00AM和12:00AM 分别执行一次
     * @param param 参数
     * @return
     * @throws Exception
     */

    @Resource
    private MemberUserLevelConfigService memberUserLevelConfigService;
    @Resource
    private MemberUserService userService;
    @Resource
    private MemberUserLevelDetailsService memberUserLevelDetailsService;
    @Resource
    private AdminUserApi adminUserApi;
    @Resource
    private MemberUserLevelOperateLogService memberUserLevelOperateService;
    @Lazy
    @Resource
    private MemberUserScoreLogService memberUserScoreLogService;
    @Override
    public String execute(String param) throws Exception {
        log.info("level default details  task running");
        //查找默认的蓝卡等级，等级为1
        LambdaQuery<MemberUserLevelConfigDO> userLevelConfigDoLambdaQuery = new LambdaQuery<>();
        userLevelConfigDoLambdaQuery.eq(MemberUserLevelConfigDO::getStatus, MemberLevelStatusEnum.ENABLED.getValue());
        userLevelConfigDoLambdaQuery.eq(MemberUserLevelConfigDO::getLevel, 1);
        List<MemberUserLevelConfigDO> todoList = memberUserLevelConfigService.selectList(userLevelConfigDoLambdaQuery);
        log.info("level default details task,  rule count :{}", todoList.size());
        if (CollectionUtils.isEmpty(todoList)) {
            return "success";
        } else {
            MemberUserLevelConfigDO memberUserLevelConfigDO=todoList.get(0);
            log.info("level default details task,  rule name :{}", memberUserLevelConfigDO.getRuleNumber());
            List<MemberUserDO> userByNoLevelDetailList = userService.getUserByNoLevelDetail();
            if (CollectionUtils.isEmpty(userByNoLevelDetailList)){
                return "success";
            }else {
                userByNoLevelDetailList.stream().forEach(memberUserDO -> {
                    setConfigDetails(memberUserDO, memberUserLevelConfigDO);
                });
            }
            log.info("定时任务完成");
        }
        return "success";
    }

    private void setConfigDetails(MemberUserDO memberUserDO, MemberUserLevelConfigDO memberUserLevelConfigDO) {
        //获取用户是否有详情
        MemberUserLevelDetailsQueryVO queryVO=new MemberUserLevelDetailsQueryVO();
        queryVO.setMemberId(memberUserDO.getId());
        PageResult<MemberUserLevelDetailsBackVO> page = memberUserLevelDetailsService.getPage(queryVO);
        AdminUserRespDTO user = adminUserApi.getUser(1L);
        List<MemberUserLevelDetailsBackVO> list = page.getList();
        if (CollectionUtils.isEmpty(list)){
        //没有详情，新增一条
        MemberUserLevelDetailsDO memberUserLevelDetailsDO = new MemberUserLevelDetailsDO();
        memberUserLevelDetailsDO.setMemberId(memberUserDO.getId());
        memberUserLevelDetailsDO.setLevel(memberUserLevelConfigDO.getLevel());
        memberUserLevelDetailsDO.setSpecificSettings(false);
        if (memberUserLevelConfigDO.getValidityPeriod() != null) {
            memberUserLevelDetailsDO.setValidityPeriod(DateUtils.getNextNDayStart(new Date(), memberUserLevelConfigDO.getValidityPeriod()));
        }
        memberUserLevelDetailsDO.setRemarks(memberUserLevelConfigDO.getDescription());
        memberUserLevelDetailsDO.setConfigId(memberUserLevelConfigDO.getId());
        memberUserLevelDetailsDO.setCreator(String.valueOf(user.getId()));
        memberUserLevelDetailsDO.setUpdater(String.valueOf(user.getId()));
        memberUserLevelDetailsService.save(memberUserLevelDetailsDO);
        log.info("设置默认等级，客户没有等级详情，增加等级详情进入升级操作，规则id：{}，用户id：{}", memberUserLevelConfigDO.getRuleNumber(), memberUserDO.getId());

        //设置用户为admin

        //插入一条日志 升级日志
        MemberUserLevelOperateLogDO memberUserLevelOperateLogDo = MemberUserLevelOperateLogDO.builder()
                .configId(memberUserLevelDetailsDO.getConfigId())
                .configName(memberUserLevelConfigDO.getName())
                .memberId(memberUserLevelDetailsDO.getMemberId())
                .operator(user.getNickname())
                .remarks(memberUserLevelDetailsDO.getRemarks())
                .level(memberUserLevelDetailsDO.getLevel())
                .operateType(1)
                .build();
        memberUserLevelOperateService.save(memberUserLevelOperateLogDo);
    }
    }
}
