package cn.iocoder.yudao.module.member.mq.producer.score;

import cn.iocoder.yudao.framework.mq.core.RedisMQTemplate;
import cn.iocoder.yudao.module.member.api.ScoreProducerApi;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.mq.message.ScoreMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author zhangfeng
 */
@Slf4j
@Component
public class ScoreProducer implements ScoreProducerApi {
    @Resource
    private RedisMQTemplate redisMQTemplate;

    @Override
    public void sendOderMessage(ScoreRuleTypeEnum scoreRuleType, Long orderId, String orderNo) {
        ScoreMessage message = ScoreMessage.builder()
                .scoreRuleType(scoreRuleType)
                .orderId(orderId)
                .orderNo(orderNo)
                .build();
        redisMQTemplate.send(message);
    }

    @Override
    public void sendRecommendMessage(ScoreRuleTypeEnum scoreRuleType, Long userId, String referralCode) {
        ScoreMessage message = ScoreMessage.builder()
                .scoreRuleType(scoreRuleType)
                .recommendUserId(userId)
                .referralCode(referralCode)
                .build();
        redisMQTemplate.send(message);
    }

    @Override
    public void sendRegisterMessage(ScoreRuleTypeEnum scoreRuleType, Long userId, String userNameZh, String userNameEn, String phone, Integer registerPlatform) {
        ScoreMessage message = ScoreMessage.builder()
                .scoreRuleType(scoreRuleType)
                .userId(userId)
                .userNameZh(userNameZh)
                .userNameEn(userNameEn)
                .phone(phone)
                .registerPlatform(registerPlatform)
                .build();
        redisMQTemplate.send(message);
    }

    @Override
    public void sendShareMessage(ScoreRuleTypeEnum scoreRuleType, Long userId, Long ruleId, String ipAddr) {
        ScoreMessage message = ScoreMessage.builder()
                .scoreRuleType(scoreRuleType)
                .userId(userId)
                .shareRuleId(ruleId)
                .triggerIp(ipAddr)
                .build();
        redisMQTemplate.send(message);
    }
}
