package cn.iocoder.yudao.module.member.service.address;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.member.dal.dataobject.address.UserAddressDO;
import cn.iocoder.yudao.module.member.vo.address.UserAddressCreateReqVO;
import cn.iocoder.yudao.module.member.vo.address.UserAddressQueryVO;
import cn.iocoder.yudao.module.member.vo.address.UserAddressUpdateReqVO;

/**
 * 会员收货地址 Service 接口
 *
 * @author 系统管理员
 */
public interface UserAddressService extends IService<UserAddressDO> {

    /**
     * 创建会员收货地址
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createUserAddress(@Valid UserAddressCreateReqVO createReqVO);

    /**
     * 更新会员收货地址
     * @param updateReqVO 更新信息
     */
    void updateUserAddress(@Valid UserAddressUpdateReqVO updateReqVO);

    /**
     * 删除会员收货地址
     * @param id 编号
     */
    void deleteUserAddress(Long id);

    /**
     * 获得会员收货地址
     * @param id 编号
     * @return 会员收货地址
     */
    UserAddressDO getUserAddress(Long id);

    /**
     * 获得会员收货地址列表
     * @param ids 编号
     * @return 会员收货地址列表
     */
    List<UserAddressDO> getUserAddressList(Collection<Long> ids);

    /**
     * 获得会员收货地址分页
     * @param page 分页查询
     * @param query 查询
     * @return 会员收货地址分页
     */
    PageResult<UserAddressDO> getUserAddressPage(UserAddressQueryVO query, PageVO page);

    /**
     * 获得会员收货地址列表, 用于 Excel 导出
     * @param query 查询
     * @return 会员收货地址列表
     */
    List<UserAddressDO> getUserAddressList(UserAddressQueryVO query);

    List<UserAddressDO> getUserAddressListByMemberId(Long memberId);
}
