package cn.iocoder.yudao.module.member.service.address;

import java.util.*;
import javax.annotation.Resource;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.ecw.api.paramValid.ParamValidatorApi;
import cn.iocoder.yudao.module.member.convert.address.UserAddressConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.address.UserAddressDO;
import cn.iocoder.yudao.module.member.dal.mysql.address.UserAddressMapper;
import cn.iocoder.yudao.module.member.enums.YesOrNoTypeEnum;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.member.vo.address.UserAddressBaseVO;
import cn.iocoder.yudao.module.member.vo.address.UserAddressCreateReqVO;
import cn.iocoder.yudao.module.member.vo.address.UserAddressQueryVO;
import cn.iocoder.yudao.module.member.vo.address.UserAddressUpdateReqVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 会员收货地址 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class UserAddressServiceImpl extends AbstractService<UserAddressMapper, UserAddressDO> implements UserAddressService {

    @Resource
    private UserAddressMapper userAddressMapper;
    @Resource
    private MemberUserService userService;
    @Resource
    private ParamValidatorApi paramValidatorApi;

    @Override
    public Long createUserAddress(UserAddressCreateReqVO createReqVO) {
        verifyUserAddress(createReqVO);
        // 插入
        UserAddressDO userAddress = UserAddressConvert.INSTANCE.convert(createReqVO);
        updateIsDefault(createReqVO, 0L);
        userAddressMapper.insert(userAddress);
        // 返回
        return userAddress.getId();
    }

    @Override
    public void updateUserAddress(UserAddressUpdateReqVO updateReqVO) {
        verifyUserAddress(updateReqVO);
        // 校验存在
        this.validateUserAddressExists(updateReqVO.getId());
        updateIsDefault(updateReqVO, updateReqVO.getId());
        // 更新
        UserAddressDO updateObj = UserAddressConvert.INSTANCE.convert(updateReqVO);
        userAddressMapper.updateById(updateObj);
    }

    // 校验参数
    private void verifyUserAddress(UserAddressBaseVO reqVO) {
        // 会员id是否存在
        //Long memberId = reqVO.getMemberId();
        //if (userService.getUser(memberId) == null) {
        //    throw exception(USER_NOT_EXISTS);
        //}
        if (StringUtils.isAnyBlank(reqVO.getAddress(), reqVO.getAreaCode(), reqVO.getName(), reqVO.getPhone())) {
            throw exception(USER_ADDRESS_FIELD_ERROR,"Address,AreaCode,Name,Phone must not null");
        }
        // 校验手机号格式
        String mobileCode = reqVO.getAreaCode() + StrUtil.COLON + reqVO.getPhone();
        paramValidatorApi.validatorMobile(mobileCode);
    }

    private void updateIsDefault(UserAddressBaseVO updateReqVO, Long id) {
        // 如果更新了默认，修改其他地址默认状态保证只有一个默认
        if (updateReqVO.getIsDefault() != null && updateReqVO.getIsDefault().equals(YesOrNoTypeEnum.YES.ordinal())) {
            for (UserAddressDO userAddressDO : getUserAddressListByMemberId(updateReqVO.getMemberId())) {
                if (!Objects.equals(userAddressDO.getId(), id) && userAddressDO.getIsDefault().equals(YesOrNoTypeEnum.YES.ordinal())) {
                    userAddressDO.setIsDefault(YesOrNoTypeEnum.NO.ordinal());
                    userAddressMapper.updateById(userAddressDO);
                }
            }
        }
    }

    @Override
    public void deleteUserAddress(Long id) {
        // 校验存在
        this.validateUserAddressExists(id);
        // 删除
        userAddressMapper.deleteById(id);
    }

    private void validateUserAddressExists(Long id) {
        if (userAddressMapper.selectById(id) == null) {
            throw exception(USER_ADDRESS_NOT_EXISTS);
        }
    }

    @Override
    public UserAddressDO getUserAddress(Long id) {
        return userAddressMapper.selectById(id);
    }

    @Override
    public List<UserAddressDO> getUserAddressList(Collection<Long> ids) {
        return userAddressMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<UserAddressDO> getUserAddressPage(UserAddressQueryVO query, PageVO page) {
        return userAddressMapper.selectPage(page, query);
    }

    @Override
    public List<UserAddressDO> getUserAddressList(UserAddressQueryVO query) {
        return userAddressMapper.selectList(query);
    }

    @Override
    public List<UserAddressDO> getUserAddressListByMemberId(Long memberId) {
        return userAddressMapper.selectList("member_id", memberId);
    }
}
