package cn.iocoder.yudao.module.member.service.levelLog;

import cn.hutool.core.lang.generator.SnowflakeGenerator;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.member.dal.dataobject.levelLog.MemberUserLevelLogDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserLevelLog.MemberUserLevelLogMapper;
import cn.iocoder.yudao.module.member.vo.userLevelLog.MemberUserLevelLogBackVO;
import cn.iocoder.yudao.module.member.vo.userLevelLog.MemberUserLevelLogCreateReq;
import cn.iocoder.yudao.module.member.vo.userLevelLog.MemberUserLevelLogQueryVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.MEMBER_ID_IS_NULL;

/**
 * 会员等级日志 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class MemberUserLevelLogServiceImpl extends AbstractService<MemberUserLevelLogMapper, MemberUserLevelLogDO> implements MemberUserLevelLogService {

    @Resource
    private MemberUserLevelLogMapper userLevelLogMapper;
    @Resource
    private SnowflakeGenerator snowflakeGenerator;


    @Override
    public PageResult<MemberUserLevelLogBackVO> getPage(MemberUserLevelLogQueryVO query) {
        int start = (query.getPageNo() - 1) * query.getPageSize();
        int size = query.getPageSize();
        List<MemberUserLevelLogBackVO> list = userLevelLogMapper.getPageRecordList(start, size, query);
        int total = userLevelLogMapper.getPageCount(query);
        return new PageResult<>(list, total, query.getPageSize(), query.getPageNo(), (total + query.getPageSize() - 1) / query.getPageSize());
    }

    @Override
    public Long createLevelLog(MemberUserLevelLogCreateReq createReq) {
        if (createReq.getConfigId() == null) {
            throw exception(MEMBER_ID_IS_NULL);
        }
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        MemberUserLevelLogDO userLevelLogDO = new MemberUserLevelLogDO();
        userLevelLogDO.setId(snowflakeGenerator.next());
        userLevelLogDO.setConfigId(createReq.getConfigId());
        userLevelLogDO.setRemarks(createReq.getRemarks());
        userLevelLogDO.setOperator(loginUser.getNickname());
        userLevelLogDO.setOperateType(createReq.getOperateType());
        userLevelLogDO.setBpmProcessId(createReq.getBpmProcessId());
        this.saveOrUpdate(userLevelLogDO);
        return userLevelLogDO.getId();
    }

}
