package cn.iocoder.yudao.module.member.service.scoreDetailReleation;

import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreDetailReleation.MemberUserScoreDetailReleationDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserScoreDetailReleation.MemberUserScoreDetailReleationMapper;
import cn.iocoder.yudao.module.member.dto.ScoreDetailChangeDto;
import cn.iocoder.yudao.module.member.enums.MemberScoreDetailReleationStatueEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 会员积分详情 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class MemberUserScoreDetailReleationServiceImpl extends AbstractService<MemberUserScoreDetailReleationMapper, MemberUserScoreDetailReleationDO> implements MemberUserScoreDetailReleationService {

    @Override
    public void updateReleation(String releationId, ScoreSourceTypeEnum sourceType, List<ScoreDetailChangeDto> detailInfos) {
        if (StringUtils.isBlank(releationId) || CollectionUtils.isEmpty(detailInfos)) {
            return;
        }
        if (sourceType.getReverseSource() != null) {
            //回退流程， 失效掉之前的关联关系
            LambdaUpdateWrapper<MemberUserScoreDetailReleationDO> updateWrapper = Wrappers.lambdaUpdate();
            updateWrapper.eq(MemberUserScoreDetailReleationDO::getReleationId, releationId);
            updateWrapper.eq(MemberUserScoreDetailReleationDO::getSourceType, sourceType.getReverseSource().getValue());
            updateWrapper.in(MemberUserScoreDetailReleationDO::getDetailId, detailInfos.stream().map(ScoreDetailChangeDto::getDetailId).collect(Collectors.toList()));
            updateWrapper.set(MemberUserScoreDetailReleationDO::getStatus, MemberScoreDetailReleationStatueEnum.NOT_AVAILABLE.getValue());
            this.update(updateWrapper);
            return;
        }
        List<MemberUserScoreDetailReleationDO> dos = detailInfos.stream().map(scoreDetailChangeDto -> {
            MemberUserScoreDetailReleationDO releationDO = new MemberUserScoreDetailReleationDO();
            releationDO.setReleationId(releationId);
            releationDO.setSourceType(sourceType.getValue());
            releationDO.setDetailId(scoreDetailChangeDto.getDetailId());
            releationDO.setScoreCount(scoreDetailChangeDto.getScoreCount());
            releationDO.setStatus(MemberScoreDetailReleationStatueEnum.AVAILABLE.getValue());
            return releationDO;
        }).collect(Collectors.toList());
        this.saveOrUpdateBatch(dos);
    }

    @Override
    public List<MemberUserScoreDetailReleationDO> listByReleationIdAndSource(String releationId, int sourceType) {
        if (StringUtils.isBlank(releationId)) {
            return Collections.emptyList();
        }
        LambdaQueryWrapper<MemberUserScoreDetailReleationDO> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MemberUserScoreDetailReleationDO::getReleationId, releationId);
        wrapper.eq(MemberUserScoreDetailReleationDO::getSourceType, sourceType);
        wrapper.eq(MemberUserScoreDetailReleationDO::getStatus, MemberScoreDetailReleationStatueEnum.AVAILABLE.getValue());
        return this.list(wrapper);
    }
}
