package cn.iocoder.yudao.module.member.service.scoreRule;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.member.controller.app.scoreRule.vo.AppScoreRuleBackDetailVO;
import cn.iocoder.yudao.module.member.controller.app.scoreRule.vo.AppScoreRuleListBackVO;
import cn.iocoder.yudao.module.member.controller.app.scoreRule.vo.AppScoreRuleListReqVO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.TransportTypeEnum;
import cn.iocoder.yudao.module.member.vo.scoreRule.*;

/**
 * 积分规则 Service 接口
 *
 * @author 系统管理员
 */
public interface ScoreRuleService extends IService<ScoreRuleDO> {

    /**
     * 创建积分规则
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createScoreRule(@Valid ScoreRuleCreateReqVO createReqVO);

    /**
     * 更新积分规则
     *
     * @param updateReqVO 更新信息
     */
    void updateScoreRule(@Valid ScoreRuleUpdateReqVO updateReqVO);

    /**
     * 删除积分规则
     *
     * @param id 编号
     */
    void deleteScoreRule(Long id);

    /**
     * 获得积分规则
     *
     * @param id 编号
     * @return 积分规则
     */
    ScoreRuleBackDetailVO getScoreRule(Long id);

    /**
     * 获得积分规则列表
     *
     * @param ids 编号
     * @return 积分规则列表
     */
    List<ScoreRuleBackVO> getScoreRuleList(Collection<Long> ids);

    /**
     * 获得积分规则分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 积分规则分页
     */
    PageResult<ScoreRuleBackVO> getScoreRulePage(ScoreRuleQueryVO query, PageVO page);

    /**
     * 获得积分规则列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 积分规则列表
     */
    List<ScoreRuleDO> getScoreRuleList(ScoreRuleQueryVO query);

    /**
     * 积分规则启用关闭
     *
     * @param scoreRuleStatusReqVO 积分规则状态
     * @return
     */
    void updateStatus(ScoreRuleStatusReqVO scoreRuleStatusReqVO);

    /**
     * 积分规则复制
     *
     * @param id
     * @return
     */
    Long copyScoreRule(Long id);

    /**
     * 积分规则延期
     *
     * @param scoreDelayReqVO 积分规则延期
     * @return
     */
    void delayScoreRule(ScoreDelayReqVO scoreDelayReqVO);

    /**
     * 根据运输方式获取已启用订单V值规则
     *
     * @param transportType
     * @return
     */
    ScoreRuleDO getEnabledOrderVScoreRuleByTransportType(Integer transportType);

    /**
     * 根据类型获取已启用积分规则（仅推荐和注册）
     * @param scoreRuleType
     * @return
     */
    ScoreRuleDO getEnableScoreRuleByType(ScoreRuleTypeEnum scoreRuleType);

    List<WarehouseTreeRegionSimpleVO> getWarehouseTreeRegionListSimple();

    /**
     * 客户端获取积分规则列表
     * @param reqVO
     * @return
     */
    List<AppScoreRuleListBackVO> appGetScoreRuleList(AppScoreRuleListReqVO reqVO);

    /**
     * 客户端获取积分规则详情
     * @param id
     * @return
     */
    AppScoreRuleBackDetailVO appGetScoreRule(Long id);

    /**
     * 获取会员功能开关状态
     * @return
     */
    Boolean getScoreRuleSwitch();

    /**
     * 设置会员功能开关
     * @param switchReqVo
     * @return
     */
    Boolean setScoreRuleSwitch(SwitchReqVo switchReqVo);

    /**
     * 获取当前时间已启用的订单V值规则
     * @return
     */
    List<ScoreRuleDO> getEnableOrderVRule();
}
