package cn.iocoder.yudao.module.member.service.userEnterprise;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.dal.mysql.user.MemberUserMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.vo.userEnterprise.*;
import cn.iocoder.yudao.module.member.dal.dataobject.userEnterprise.UserEnterpriseDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.member.convert.userEnterprise.UserEnterpriseConvert;
import cn.iocoder.yudao.module.member.dal.mysql.userEnterprise.UserEnterpriseMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 会员公司信息 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class UserEnterpriseServiceImpl extends AbstractService<UserEnterpriseMapper, UserEnterpriseDO> implements UserEnterpriseService {

    @Resource
    private UserEnterpriseMapper userEnterpriseMapper;
    @Resource
    private MemberUserMapper memberUserMapper;

    @Override
    public Long createUserEnterprise(UserEnterpriseCreateReqVO createReqVO) {
        UserEnterpriseDO orgUserEnterprise = userEnterpriseMapper.selectOne(new LambdaQueryWrapper<UserEnterpriseDO>().eq(UserEnterpriseDO::getUserId, createReqVO.getUserId()));
        // 插入
        UserEnterpriseDO userEnterprise = UserEnterpriseConvert.INSTANCE.convert(createReqVO);
        if (Objects.nonNull(orgUserEnterprise)){
            userEnterprise.setId(orgUserEnterprise.getId());
            userEnterpriseMapper.updateById(userEnterprise);
        }else {
            userEnterpriseMapper.insert(userEnterprise);
        }
        // 返回
        return userEnterprise.getId();
    }

    @Override
    public void updateUserEnterprise(UserEnterpriseUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateUserEnterpriseExists(updateReqVO.getId());
        // 更新
        UserEnterpriseDO updateObj = UserEnterpriseConvert.INSTANCE.convert(updateReqVO);
        userEnterpriseMapper.updateById(updateObj);
    }

    @Override
    public void deleteUserEnterprise(Long id) {
        // 校验存在
        this.validateUserEnterpriseExists(id);
        // 删除
        userEnterpriseMapper.deleteById(id);
    }

    private void validateUserEnterpriseExists(Long id) {
        if (userEnterpriseMapper.selectById(id) == null) {
            throw exception(USER_ENTERPRISE_NOT_EXISTS);
        }
    }

    @Override
    public UserEnterpriseDO getUserEnterprise(Long id) {
        return userEnterpriseMapper.selectOne(new LambdaQueryWrapper<UserEnterpriseDO>().eq(UserEnterpriseDO::getUserId,id));
    }


    @Override
    public List<UserEnterpriseDO> getUserEnterpriseList(Collection<Long> ids) {
        return userEnterpriseMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<UserEnterpriseDO> getUserEnterprisePage(UserEnterpriseQueryVO query, PageVO page) {
        return userEnterpriseMapper.selectPage(page, query);
    }

    @Override
    public List<UserEnterpriseDO> getUserEnterpriseList(UserEnterpriseQueryVO query) {
        return userEnterpriseMapper.selectList(query);
    }
}
