package cn.iocoder.yudao.module.member.service.userEnterpriseAuth;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.member.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.member.enums.UserAuditStatusEnum;
import cn.iocoder.yudao.module.member.enums.UserOperationLogTypeEnum;
import cn.iocoder.yudao.module.member.service.userOperationLog.UserOperationLogService;
import cn.iocoder.yudao.module.member.vo.userOperationLog.UserOperationLogCreateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.vo.userEnterpriseAuth.*;
import cn.iocoder.yudao.module.member.dal.dataobject.userEnterpriseAuth.UserEnterpriseAuthDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.member.convert.userEnterpriseAuth.UserEnterpriseAuthConvert;
import cn.iocoder.yudao.module.member.dal.mysql.userEnterpriseAuth.UserEnterpriseAuthMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.util.servlet.ServletUtils.getClientIP;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 用户企业认证 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class UserEnterpriseAuthServiceImpl extends AbstractService<UserEnterpriseAuthMapper, UserEnterpriseAuthDO> implements UserEnterpriseAuthService {

    @Resource
    private UserEnterpriseAuthMapper userEnterpriseAuthMapper;

    @Resource
    private UserOperationLogService userOperationLogService;

    @Override
    public Long createUserEnterpriseAuth(UserEnterpriseAuthCreateReqVO createReqVO, boolean isAdminManagerUpload) {


        //已存在
        if (userEnterpriseAuthMapper.selectCount(UserEnterpriseAuthDO::getUserId, createReqVO.getUserId()) > 0) {
            throw exception(ErrorCodeConstants.USER_CARD_AUTH_EXISTS);
        }

        // 插入
        UserEnterpriseAuthDO userEnterpriseAuth = UserEnterpriseAuthConvert.INSTANCE.convert(createReqVO);
        if (isAdminManagerUpload) {
            userEnterpriseAuth.setStatus(UserAuditStatusEnum.AUDIT_SUCCESS.getValue());
            userEnterpriseAuth.setAuditTime(new Date());
        } else {
            userEnterpriseAuth.setStatus(UserAuditStatusEnum.AUDITING.getValue());
        }
        userEnterpriseAuthMapper.insert(userEnterpriseAuth);

        //记录日志
        UserOperationLogCreateReqVO userOperationLogCreateReqVO = new UserOperationLogCreateReqVO();
        userOperationLogCreateReqVO
                .setUserId(Long.parseLong(userEnterpriseAuth.getCreator()))
                .setType(UserOperationLogTypeEnum.AUTHENTIFICATION.getValue())
                .setTitle(UserOperationLogTypeEnum.AUTHENTIFICATION.getDesc())
                .setIp(getClientIP())
                .setContent("企业认证已提交");

        userOperationLogService.createUserOperationLog(userOperationLogCreateReqVO);


        // 返回
        return userEnterpriseAuth.getId();
    }

    @Override
    public void updateUserEnterpriseAuth(UserEnterpriseAuthUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateUserEnterpriseAuthExists(updateReqVO.getId());
        // 更新
        UserEnterpriseAuthDO updateObj = UserEnterpriseAuthConvert.INSTANCE.convert(updateReqVO);
        userEnterpriseAuthMapper.updateById(updateObj);
    }

    @Override
    public void deleteUserEnterpriseAuth(Long id) {
        // 校验存在
        this.validateUserEnterpriseAuthExists(id);
        // 删除
        userEnterpriseAuthMapper.deleteById(id);
    }

    private void validateUserEnterpriseAuthExists(Long id) {
        if (userEnterpriseAuthMapper.selectById(id) == null) {
            throw exception(USER_ENTERPRISE_AUTH_NOT_EXISTS);
        }
    }

    @Override
    public UserEnterpriseAuthDO getUserEnterpriseAuth(Long id) {
        return userEnterpriseAuthMapper.selectById(id);
    }

    @Override
    public List<UserEnterpriseAuthDO> getUserEnterpriseAuthList(Collection<Long> ids) {
        return userEnterpriseAuthMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<UserEnterpriseAuthDO> getUserEnterpriseAuthPage(UserEnterpriseAuthQueryVO query, PageVO page) {
        return userEnterpriseAuthMapper.selectPage(page, query);
    }

    @Override
    public List<UserEnterpriseAuthDO> getUserEnterpriseAuthList(UserEnterpriseAuthQueryVO query) {
        return userEnterpriseAuthMapper.selectList(query);
    }
}
