package cn.iocoder.yudao.module.member.service.userOperationLog;

import java.util.*;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.vo.userOperationLog.*;
import cn.iocoder.yudao.module.member.dal.dataobject.userOperationLog.UserOperationLogDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.member.convert.userOperationLog.UserOperationLogConvert;
import cn.iocoder.yudao.module.member.dal.mysql.userOperationLog.UserOperationLogMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 用户操作日志 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class UserOperationLogServiceImpl extends AbstractService<UserOperationLogMapper, UserOperationLogDO> implements UserOperationLogService {

    @Resource
    private UserOperationLogMapper userOperationLogMapper;

    @Override
    public Long createUserOperationLog(UserOperationLogCreateReqVO createReqVO) {
        // 插入
        UserOperationLogDO userOperationLog = UserOperationLogConvert.INSTANCE.convert(createReqVO);
        userOperationLogMapper.insert(userOperationLog);
        // 返回
        return userOperationLog.getId();
    }

    @Override
    public void updateUserOperationLog(UserOperationLogUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateUserOperationLogExists(updateReqVO.getId());
        // 更新
        UserOperationLogDO updateObj = UserOperationLogConvert.INSTANCE.convert(updateReqVO);
        userOperationLogMapper.updateById(updateObj);
    }

    @Override
    public void deleteUserOperationLog(Long id) {
        // 校验存在
        this.validateUserOperationLogExists(id);
        // 删除
        userOperationLogMapper.deleteById(id);
    }

    private void validateUserOperationLogExists(Long id) {
        if (userOperationLogMapper.selectById(id) == null) {
            throw exception(USER_OPERATION_LOG_NOT_EXISTS);
        }
    }

    @Override
    public UserOperationLogDO getUserOperationLog(Long id) {
        return userOperationLogMapper.selectById(id);
    }

    @Override
    public List<UserOperationLogDO> getUserOperationLogList(Collection<Long> ids) {
        return userOperationLogMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<UserOperationLogDO> getUserOperationLogPage(UserOperationLogQueryVO query, PageVO page) {
        return userOperationLogMapper.selectPage(page, query);
    }

    @Override
    public List<UserOperationLogDO> getUserOperationLogList(UserOperationLogQueryVO query) {
        return userOperationLogMapper.selectList(query);
    }
}
