package cn.iocoder.yudao.module.member.util;

import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.module.member.dal.mysql.ScoreRuleShareRecord.ScoreRuleShareRecordMapper;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserLevel.MemberUserLevelConfigMapper;
import cn.iocoder.yudao.module.member.dal.mysql.scoreRule.ScoreRuleMapper;
import com.alibaba.excel.util.DateUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.infra.enums.ErrorCodeConstants.GET_LOCK_FAILED;

/**
 * 根据规则生成各种code
 */
@Component
public class MenberUserLevelGenCodeUtils {

    @Resource
    private RedisHelper redisHelper;
    @Resource
    private RedissonClient redissonClient;
    @Resource
    private MemberUserLevelConfigMapper memberUserLevelConfigMapper;


    /**
     * 生成积分规则编号
     */
    public String generateMemberLevelCode() {
        // 编号规则为MB+年份+5位数，例如MB2400001
        String key = "memberLevel:max:number";
        StringBuilder code = new StringBuilder();
        code.append("MB");
        code.append(DateUtils.format(new Date(), "yy"));
        Long codeNum;
        if (redisHelper.hasKey(key)) {
            codeNum = redisHelper.incrBy(key, 1);
        } else {
            RLock lock = redissonClient.getLock("next:memberLevel:code:lock");
            try {
                boolean lockSuccess = lock.tryLock(2, 2, TimeUnit.SECONDS);
                if (!lockSuccess) {
                    throw exception(GET_LOCK_FAILED);
                }
                String currentMaxMemberUserLevelRuleNumber = memberUserLevelConfigMapper.getCurrentMaxMemberUserLevelRuleNumber();
                if (currentMaxMemberUserLevelRuleNumber == null){
                    codeNum = 1L;
                } else {
                    codeNum = Long.parseLong(currentMaxMemberUserLevelRuleNumber.substring(4)) + 1;
                }
                redisHelper.set(key, String.valueOf(codeNum),10, TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                throw exception(GET_LOCK_FAILED);
            } finally {
                lock.unlock();
            }
        }
        // 获得5位序列号，不足位前面补0
        codeNum = codeNum % 100000;
        code.append(String.format("%05d", codeNum));
        return code.toString();
    }
}
