package cn.iocoder.yudao.module.member.util;

import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.mysql.ScoreRuleShareRecord.ScoreRuleShareRecordMapper;
import cn.iocoder.yudao.module.member.dal.mysql.scoreRule.ScoreRuleMapper;
import com.alibaba.excel.util.DateUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.infra.enums.ErrorCodeConstants.GET_LOCK_FAILED;

/**
 * 根据规则生成各种code
 */
@Component
public class ScoreRuleGenCodeUtils {

    @Resource
    private RedisHelper redisHelper;
    @Resource
    private RedissonClient redissonClient;
    @Resource
    private ScoreRuleMapper scoreRuleMapper;
    @Resource
    private ScoreRuleShareRecordMapper scoreRuleShareRecordMapper;

    /**
     * 生成积分规则编号
     */
    public String generateScoreRuleCode() {
        // 编号规则为PR+年份+5位数，例如PR2400001
        String key = "scoreRule:max:number";
        StringBuilder code = new StringBuilder();
        code.append("PR");
        code.append(DateUtils.format(new Date(), "yy"));
        Long codeNum;
        if (redisHelper.hasKey(key)) {
            codeNum = redisHelper.incrBy(key, 1);
        } else {
            RLock lock = redissonClient.getLock("next:scoreRule:code:lock");
            try {
                boolean lockSuccess = lock.tryLock(2, 2, TimeUnit.SECONDS);
                if (!lockSuccess) {
                    throw exception(GET_LOCK_FAILED);
                }
                String currentMaxScoreRuleCode = scoreRuleMapper.getCurrentMaxScoreRuleCode();
                if (currentMaxScoreRuleCode == null){
                    codeNum = 1L;
                } else {
                    codeNum = Long.parseLong(currentMaxScoreRuleCode.substring(4)) + 1;
                }
                redisHelper.set(key, String.valueOf(codeNum),10, TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                throw exception(GET_LOCK_FAILED);
            } finally {
                lock.unlock();
            }
        }
        // 获得5位序列号，不足位前面补0
        codeNum = codeNum % 100000;
        code.append(String.format("%05d", codeNum));
        return code.toString();
    }
    /**
     * 生成分享记录编号
     */
    public String generateShareRecordCode() {
        // 编号规则：S+年份+月份+5位数，S240100005
        String key = "shareRecord:max:number";
        StringBuilder code = new StringBuilder();
        code.append("S");
        code.append(DateUtils.format(new Date(), "yyMM"));
        Long codeNum;
        if (redisHelper.hasKey(key)) {
            codeNum = redisHelper.incrBy(key, 1);
        } else {
            RLock lock = redissonClient.getLock("next:shareRecord:code:lock");
            try {
                boolean lockSuccess = lock.tryLock(2, 2, TimeUnit.SECONDS);
                if (!lockSuccess) {
                    throw exception(GET_LOCK_FAILED);
                }
                String currentMaxShareRecordCode = scoreRuleShareRecordMapper.getCurrentMaxShareRecordCode();
                if (currentMaxShareRecordCode == null){
                    codeNum = 1L;
                } else {
                    codeNum = Long.parseLong(currentMaxShareRecordCode.substring(5)) + 1;
                }
                redisHelper.set(key, String.valueOf(codeNum),10, TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                throw exception(GET_LOCK_FAILED);
            } finally {
                lock.unlock();
            }
        }
        // 获得5位序列号
        codeNum = codeNum % 100000;
        code.append(String.format("%05d", codeNum));
        return code.toString();
    }
    public void historyScoreRuleCodeFlush() {
        long start = 1L;
        redisHelper.delete("scoreRule:max:number");
        List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectList();
        for (ScoreRuleDO scoreRuleDO : scoreRuleDOS) {
            StringBuilder code = new StringBuilder();
            code.append("PR");
            code.append(DateUtils.format(new Date(), "yy"));
            code.append(String.format("%05d", start));
            scoreRuleDO.setCode(code.toString());
            scoreRuleMapper.updateById(scoreRuleDO);
            start += 1;
        }
    }
}
